﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

namespace Bonfire
{
    public enum State
    {
        Died,
        Walking,
        AttackR1,
        Rolling,
        Jump
    };

    public class PlayerCtrl : MonoBehaviour
    {
        [SerializeField] Text debugText = null;

        [SerializeField] float joystickDead = 0.3f;
        float inputHorizontalStickL = 0f;
        float inputVerticalStickL = 0f;

        State state = State.Walking;        // 現在の状態
        State nextState = State.Walking;    // 次の状態
        bool isStateEnd = true;

        // 走っている、全力疾走しているフラグ
        public bool isRunning = false;
        public bool isSprint = false;

        // xボタンの入力が、ローリングからダッシュになるまでの時間を計算するための変数
        float timeSinceInputFire1 = 0f;

        PlayerStatus status = null;
        PlayerMove playerMove = null;
        HudController hudController = null;

        //RIGIDBODY
        PlayerMoveRigidbody playerMoveRigidbody = null;


        void Start()
        {
            status = GetComponent<PlayerStatus>();
            playerMove = GetComponent<PlayerMove>();
            hudController = FindObjectOfType<HudController>();

            playerMoveRigidbody = GetComponent<PlayerMoveRigidbody>();
        }

        void Update()
        {
            UpdateTimer();
            inputHorizontalStickL = Input.GetAxis("Horizontal Stick-L");
            inputVerticalStickL = Input.GetAxis("Vertical Stick-L") * -1f; // 上下が逆なので -1 をかけておく

            // 現在の状態の処理
            switch (state)
            {
                case State.Walking:
                    Walking();
                    break;
                case State.AttackR1:
                    AttackR1();
                    break;
                case State.Rolling:
                    Rolling();
                    break;
                case State.Jump:
                    Jump();
                    break;
            }

            // ボタンの処理は、ここに足していく
            if (Input.GetButtonDown("Fire_Share"))
            {
                hudController.toggleHudLeft();
            }

            // 加速処理
            if (Input.GetButton("Fire_1"))
            {
                isRunning = true;
            }
            if (Input.GetButtonUp("Fire_1"))
            {
                isRunning = false;
            }

            if (Input.GetButtonDown("Fire_0"))
            {
                if (state != State.Jump)
                {
                    nextState = State.Jump;
                }
            }


            // 全力疾走処理
            if (Input.GetButtonDown("Fire_L3"))
            {
                isSprint = isSprint ? false : true;
            }

            // 次の状態に遷移できるようになった時の処理
            if (isStateEnd)
            {
                State prevState = state;
                // StartXX 内で nextState が書き換えられるので、ここで state を更新しておく必要がある
                state = nextState;
                switch (nextState)
                {
                    case State.Walking:
                        StartWalking(prevState);
                        break;
                    case State.AttackR1:
                        StartAttackR1(prevState);
                        break;
                    case State.Rolling:
                        StartRolling(prevState);
                        break;
                    case State.Jump:
                        StartJump(prevState);
                        break;
                }
            }

            if (debugText != null)
            {
                debugText.text =
                   string.Format("state: {0}\nnexState: {1}\nisStateEnd: {2}\nisGrounded: {3}",
                                state,
                                nextState,
                                isStateEnd,
                                status.isGrounded
                               );
            }

        }

        //
        // コントローラーの入力を返すためのメソッド
        //
        public Vector3 GetPlayerDirection()
        {
            return Vector3.Normalize(new Vector3(inputHorizontalStickL, 0, inputVerticalStickL));
        }

        public float GetMagunitudeStickL()
        {
            float ret = new Vector3(inputHorizontalStickL, 0, inputVerticalStickL).magnitude;
            return ret;
        }

        public bool IsInputStickL()
        {
            float h = inputHorizontalStickL;
            float v = inputVerticalStickL;
            bool ret = ((h > joystickDead) || (h < -joystickDead) || (v > joystickDead) || (v < -joystickDead));
            return ret;
        }

        //
        // 状態(State)を開始するときに行う処理 
        // 
        private void StartWalking(State prevState)
        {
            StateStartCommon();
        }

        private void StartAttackR1(State prevState)
        {
            StateStartCommon();
            status.isAttackR1 = true;
            nextState = prevState;
        }

        private void StartRolling(State prevState)
        {
            StateStartCommon();
            status.isRolling = true;
            nextState = State.Walking;
        }

        private void StartJump(State prevState)
        {
            StateStartCommon();
            status.isJumping = true;
            nextState = State.Walking;

            // PlayerMoveRigidbody の StartJump なので注意！！
            playerMoveRigidbody.StartJump();
        }

        //
        // 状態(State) の処理
        // 基本的には、状態が終わるのを待つだけ
        //
        private void Walking()
        {
        }

        void AttackR1()
        {
            if (!status.isAttackR1)
                ChangeState(nextState);
        }

        private void Rolling()
        {
            if (!status.isRolling)
                ChangeState(nextState);
        }

        private void Jump()
        {
            if (!status.isJumping)
                ChangeState(nextState);
        }

        //
        // 共通処理のメソッド
        //
        private void StateStartCommon()
        {
            //Debug.Log("StateStartCommon()");
            status.isDied = false;
            status.isAttackR1 = false;
            status.isRolling = false;
            status.isJumping = false;

            if (state == State.Walking)
            {
                isStateEnd = true;
            }
            else
            {
                isStateEnd = false;
            }
        }

        void ChangeState(State newStatus)
        {
            //Debug.Log("ChangeState(" + newStatus + ") called");
            nextState = newStatus;
            isStateEnd = true;
        }

        //
        // その他のメソッド
        //
        private void UpdateTimer()
        {
            timeSinceInputFire1 += Time.deltaTime;
        }
    }

}

